import { reactive, computed } from 'vue'
import * as api from '@/service/order'

export const state = reactive({
  productList: [] as any[],
  brandList: [] as any[],
  modelList: [] as any[],
  form: {
    product_id: '',
    subscribe_type: '' as 'year' | 'month',
    discount_code: '',
    brand_id: '',
    phone_id: '',
    email: '',
    from: '',
    invitor: '',
  },
  discount: null as any,
})

const selectedProduct = computed(() =>
  state.productList.find(item => item.id === state.form.product_id)
)

const cost = computed(() => {
  if (!selectedProduct.value) return ''
  const { amount_year, amount_month, amount_month_open } = selectedProduct.value
  const { discount_amount = 0 } = state.discount || {}
  switch (state.form.subscribe_type) {
    case 'year':
      return (+amount_year - discount_amount).toFixed(2)
    case 'month':
      return (+amount_month + +amount_month_open - discount_amount).toFixed(2)
    default:
      return ''
  }
})

const modelName = computed(
  () =>
    state.modelList.find(item => item.id === state.form.phone_id)?.name || ''
)

export const getters = reactive({
  selectedProduct,
  cost,
  modelName,
})

export function resetState() {
  state.modelList = []
  state.form = {} as any
  state.discount = null
}

export async function initProducts() {
  if (!state.productList.length) {
    state.productList = (await api.getProductList()).results
  }
  state.form.product_id = state.productList[0].id
}

export async function getBrandList() {
  state.brandList = (await api.getBrands()).results
}

export async function getModelList() {
  state.modelList = (await api.getModels(state.form.brand_id)).results
}
